# Spatially structured perisomatic inhibition organized along polarized fast-spiking interneuron axons


### How to setup the python environment

It is recommended to use [Anaconda](https://www.continuum.io/downloads) to create a virtual environment with the
 necessary dependencies. With anaconda, it is a one button install. After cloning the repository and changing into the root folder, type
```
conda env create -f environment.yml
```

This creates an environment called interneuron_polarity, which can be activated via
```bash
conda activate interneuron_polarity
```

To run the scripts, you also need to make the repository folder available in the python path (requires conda--build)
```bash
conda develop .
```

### How to run the simulation

The entire process is split in three main scripts: run, analysis and plotting. We use [pypet](https://pypet
.readthedocs.io/en/latest/) to organize our simulations and save intermediate results. 

#### Simulate the spatial networks
The simulation is setup and run via
```bash
cd scripts/spatial_network/perlin_map
python run_simulation_perlin_map.py
```
This script also contains the relevant parameter settings such as network size or the form of the interneuron
 axons. To allow exceution of the script in a reasonable time, we reduced the number of input maps that are tested
 . In the script, you find instructions to make the full run. After the script has successfully finished, you find
  the output in the `data` folder. 

#### Analysis of the network activity

A preliminary analysis of the simulated data is done by running 
```bash
python preliminary_analysis_perlin_map.py
```
To run both simulation and preliminary analysis 
```bash
python run_and_analyze_perlin.py
```

#### Generate plots

The final analysis and figure generation is done by 
```bash
python final_analysis_and_plotting_perlin_map.py
```

#### Supplemental figure
For the supplement figure, a pinwheel map is used as an alternative input map. The run, analyse and plot scripts are
 organized in the same manner but found in the folder `scripts/spatial_network/supplement_pinwheel_map`. 

### Additional Ressources

The simulation saves the generated data in a hdf5-file. Since the full parameter exploration used for the model section of the paper requires significant time to run, the generated data set can also be downloaded from the following location. When the pre-simulated dataset is used, only the figure script needs to be executed to get the results of the data analysis. Note that the save file will be overwritten when the simulation is started again.

[Download location](https://gin.g-node.org/Moritz/spatially_structured_perisomatic_inhibition_along_polarized_interneurons) with following content:
- Full perlin data set: spatial_network_perlin_map.hdf5
- Full pinwheel data set: spatial_network_pinwheel_map.hdf5
- Pregenerated pinwheel maps: precalculated_pinwheel_maps.hdf5
